uses getdata,graph,crt,dos;

type myreal=single; { a bit faster }

var gravity:myreal;  { acceleration of gravity }
var walldens:myreal; { density of smoothed atoms on walls }
const maxn=1000;     { max # of atoms }

type vector=record x,y:myreal; END;

type st15=string[15];
type st31=string[31];
type st79=string[179];

var r,v,a:array[1..maxn] of vector; { configuration, velocities, accelerations }
var rt,ri,incirc:vector; { trial position, r[i], random vector }
var scr:array[1..maxn] of record ix,iy:word; END; { screen coordinates: to erase old atom positions }
const n:integer=64; { # of atoms }
var ii,i,j,k,rad,maxrad,nrep:integer;
var L,Lh,Lorig:myreal; { box size, Lh=L/2, Lorig=starting value of L }
var T:myreal; { temperature as NVT parameter }
var Tk,Tm,TT:myreal; { Tk=kinetic T, Tm is summed during a sweep, TT is auxiliary }
const Tmax:myreal=2.593743;
var deltaU:myreal; { MC: Utrial-U }
var bag:myreal; { Creutz daemon bag }
var ff,vv,rr,xx,maxvv:myreal;

type bctype=(box,slit,cyclic); { boundary conditions }
var bc:bctype;

type cfgtype=(gas,liquid,crystal,defect,vacation,intersticial,dump); { initial cfg. }
const cfg:cfgtype=gas;

var dstat:(half,one{,two},Lhalf,auto,fixed); { status of MC d moves }
var dadj:myreal; { displacement for MC: automatically adjustable }
var d:myreal; { displacement for MC: actual }
var accepted:word; { # of accepted MC moves }
var ar:myreal; { acceptance ratio }

const acc:myreal=0.3333333; { acc ratio for auto set }
const tau:myreal=1; { MD thermostat time constant }
const range:myreal=2.25; { range^2 for counting neighbors (hotkey `n') }

const h:myreal=0.01; { MD timestep }
const hauto:boolean=false; { status of MD timestep }
const hadj:myreal=0.01; { MD timestep: automatically adjustable }
const P:myreal=0; { pressure, as measured from the virial of force (only if pressure) }
var pressure:boolean;

var md:boolean; { true: MD, false:MC }
var constt:boolean; { true:NVT, false:NVE }
const quiet:boolean=false;

var sx,sy:myreal; { scaling for the screen output }
var aspx,aspy:word; { pixel aspect ratio }
var col,graphdriver,graphmode:integer;

var repaint,updatepanel:boolean;

var ch:char;
var st:st15;
var sample,time0,time:myreal; { to control measure sampling rate, in seconds }
var sweeps:integer; { # of sweeps / one cycle (lasting ~ sample s) }

label escape;

{ current time in s }
PROCEDURE ctime;
var h,m,s,ss:word;
BEGIN
gettime(h,m,s,ss);
time:=ss*0.01+s+m*60.0+h*3600.0;
END;

{ atom-atom potential u(r)=r^8-r^4 (2D Lennard-Jones), as the FUNCTION of rr=r^2 }
FUNCTION u(rr:myreal):myreal;
BEGIN
rr:=1/sqr(rr);
u:=sqr(rr)-rr;
END;

{ site-site forces (to be multiplied by vector r), as the FUNCTION of rr=r^2 }
FUNCTION f(rr:myreal):myreal;
var r4:myreal;
BEGIN
r4:=sqr(rr);
f:=(8/r4-4)/rr/r4;
END;

{$f+}
{ attractive walls: site-wall potential }
FUNCTION uwalla(r:myreal):myreal;
var rr:myreal;
BEGIN
if r<=0 then uwalla:=-1e30*r
else BEGIN
  rr:=sqr(r);
  uwalla:=(walldens*0.375/sqr(rr)-walldens*0.5)/(r*rr); END;
END;

{ attractive walls: site-wall force (to be multiplied by vector r) }
FUNCTION fwalla(r:myreal):myreal;
var rr:myreal;
BEGIN
rr:=sqr(sqr(r));
fwalla:=(walldens*2.625/rr-walldens*1.5)/rr;
END;

{ repulsive walls: site-wall potential }
FUNCTION uwallr(r:myreal):myreal;
var rr:myreal;
BEGIN
if r<=0 then uwallr:=-1e30*r
else BEGIN
  rr:=sqr(r);
  uwallr:=(walldens*0.375/sqr(rr))/(r*rr); END;
END;

{ repulsive walls: site-wall force (to be multiplied by vector r) }
FUNCTION fwallr(r:myreal):myreal;
var rr:myreal;
BEGIN
rr:=sqr(sqr(r));
fwallr:=(walldens*2.625/rr)/rr;
END;

type walltype=FUNCTION(r:myreal):myreal;
var uwallx,fwallx,uwally,fwally,uwallxL,fwallxL,uwallyL,fwallyL:walltype;
{f-}

{ squared vector in nearest image convention }
FUNCTION sqrni(r:myreal):myreal;
BEGIN
sqrni:=sqr(Lh-abs(Lh-abs(r)));
END;

{ support to print the hotkey panel }
var atx,aty:integer;
const advy=25;

{ support to print the hotkey panel }
PROCEDURE keyhelp(key,msg:st31);
BEGIN
if quiet then exit;
setcolor(yellow);
outtextxy(atx,aty,key);
setcolor(lightcyan);
outtextxy(atx,aty+10,msg);
inc(aty,advy);
END;

{ support to print info on variables }
PROCEDURE info(a:st15; x:myreal; c,y:word);
BEGIN
if quiet then exit;
y:=y*19;
if x=0 then st:='0.0' else str(x:4:3+ord(x<0.01),st);
setcolor(c);
settextstyle(0,0,2);
bar(atx,y,getmaxx,y+15);
outtextxy(atx,y,a+st);
END;

{ to draw in the simulation square only }
PROCEDURE insquare;
BEGIN
if quiet then exit;
setviewport(1,1,round(sx*L)-1,round(sy*L)-1,clipon);
END;

{ to draw in the whole screen }
PROCEDURE fullscreen;
BEGIN
if quiet then exit;
setviewport(0,0,getmaxx,getmaxy,clipoff);
END;

{ print the current gravity info }
PROCEDURE prtgravity(add:myreal);
BEGIN
if bc<>cyclic then BEGIN
  gravity:=gravity+add;
  if abs(gravity)<0.001 then gravity:=0;
  aty:=40;
  fullscreen;
  info(' g=',gravity,white,3); END;
END;

{ draws walls: dotted=penetrable (cyclic)
               solid white=attractive
               solid red=repulsive }
PROCEDURE drawwalls;
BEGIN
fullscreen;
if bc=box then if @uwallx=@uwalla then setcolor(white) else setcolor(lightred)
else BEGIN setlinestyle(dottedln,0,1); setcolor(lightgray); END;
line(0,0,0,round(sy*L));
if bc=box then if @uwallxL=@uwalla then setcolor(white) else setcolor(lightred);
line(round(sx*L),0,round(sx*L),round(sy*L));

setlinestyle(solidln,0,1);
if bc<>cyclic then if @uwally=@uwalla then setcolor(white) else setcolor(lightred)
else BEGIN setlinestyle(dottedln,0,1); setcolor(lightgray); END;
line(0,0,round(sx*L),0);
if bc<>cyclic then if @uwallyL=@uwalla then setcolor(white) else setcolor(lightred);
line(0,round(sy*L),round(sx*L),round(sy*L));
END;

{ support to print the help screen }
PROCEDURE hh(s:st79);
BEGIN
outtextxy(3,aty,s);
inc(aty,13);
END;

{ prints the panel with variable info and sets min and max T and some other things }
PROCEDURE infopanel(all:boolean);
BEGIN
fullscreen;

if constt then st:='NVT' else st:='NVE';
if md then st:=st+' MD   ' else st:=st+' MC   ';
info(st,0,yellow,0);

if T<9.51337e-4 then T:=9.51337e-4;
if T>Tmax then T:=Tmax;
if constt then bag:=T;
Lh:=L/2;

info(' T=',T,8+7*ord(constt),1);

if md then BEGIN
  if hauto then BEGIN
    i:=lightred; h:=hadj END
  else i:=white;
  info('dt=',h,i,4);
  END
else BEGIN
  if dstat=auto then BEGIN
    i:=lightred; d:=dadj; END
  else i:=white;
  info('dr=',d,i,4); END;

if all then BEGIN
  if md then info('Tk=',Tm,lightcyan,2)
  else info('Td=',Tm,lightcyan-3*ord(constt),2);
  info('ar=',ar,lightcyan-3*ord(md),6);
  END;

insquare;
END;

{ calculates screen coordinates of atoms }
PROCEDURE setscr;
BEGIN
for i:=1 to n do with r[i] do with scr[i] do BEGIN
  ix:=round(x*sx); iy:=round(y*sy);
  END;
END;

{ draws all atoms, using scr[] }
PROCEDURE drawcfg;
BEGIN
if quiet then exit;
setcolor(white);
for i:=1 to n do with scr[i] do circle(ix,iy,rad);
END;

{ redraw everything }
PROCEDURE redraw;
BEGIN
if quiet then exit;
fullscreen;
clearviewport;
setfillstyle(1,black);
drawwalls;
setcolor(white);
atx:=getmaxx*10 div 13;
aty:=getmaxy-advy*13-17;
settextstyle(0,0,1);
keyhelp('h H ?     q','help      quiet');
keyhelp('m','toggle MC/MD');
keyhelp('e','toggle NVE/NVT');
if bc<>cyclic then keyhelp('t T - +   g G','change T  gravity')
else keyhelp('t T - +','change T');
if bc=box then keyhelp('x X y Y','attr/repuls walls')
else if bc=slit then keyhelp('y Y','attr/repuls walls');
keyhelp('d D','change dr or dt');
keyhelp('c C','trajectory color');
keyhelp('s S','atom size');
keyhelp('l L       o','box size  restore');
keyhelp('p','toggle Pressure');
keyhelp('n         cursors','neighbors move cfg');
keyhelp('SPACE     r',      'freeze/go redraw');
keyhelp('=         ESC','set parms exit');
prtgravity(0);
infopanel(false);
repaint:=not md;
if repaint then drawcfg;
END;

{ resize the simulation box + redraw }
PROCEDURE resize(newL:myreal);
var i:integer;
var q:myreal;
BEGIN
if newL>Lorig then newL:=Lorig;
if newL<sqrt(n/3) then newL:=sqrt(n/3);
q:=newL/L;
L:=newL;
for i:=1 to n do with r[i] do BEGIN x:=x*q; y:=y*q; END;
setscr;
redraw;
END;

{ finish, with optional ASCII dump of cfg }
PROCEDURE finish;
var f:text;
var i:integer;
BEGIN
closegraph;
if yes('write configuration to file SHOWMCMD.DUP') then BEGIN
  assign(f,'showmcmd.dup'); rewrite(f);
  writeln(f,n,' n');
  writeln(f,L:0:5,' L');
  writeln(f,T:0:3,' T');
  writeln(f,h:0:3,' h');
  writeln(f,Tmax:0:3,' Tmax');
  writeln(f,dadj:0:3,' dadj');
  writeln(f,walldens:0:3,' walldens');
  writeln(f,ord(bc),' bc');
  writeln(f,'     x         y');
  for i:=1 to n do with r[i] do writeln(f,x:9:5,y:10:5);
  close(f); END;
halt;
END;

{ read ASCII dump of cfg }
PROCEDURE readcfg;
var f:text;
var i:integer;
BEGIN
assign(f,'showmcmd.dup'); reset(f);
readln(f,n);
readln(f,L);
readln(f,T);
readln(f,h);
readln(f,Tmax);
readln(f,dadj);
readln(f,walldens);
readln(f,i); bc:=bctype(i);
readln(f);
for i:=1 to n do with r[i] do readln(f,x,y);
close(f);
END;

{ the help screen (hotkey h) }
PROCEDURE help;
BEGIN
clearviewport;
aty:=2;
settextstyle(0,0,1);
setcolor(lightgreen);
hh('SYMBOLS');
setcolor(white);
hh('NVE MC  Creutz Monte Carlo (microcanonical ensemble)');
hh('NVT MC  Metropolis Monte Carlo (canonical ensemble)');
hh('NVE MD  microcanonical molecular dynamics');
hh('NVT MD  molecular dynamics with friction thermostat');
hh('T       temperature (as parameter of NVT ensembles)');
hh('Tk      temperature calculated in MD from kinetic energy');
hh('Td      temperature calculated in NVE MC from averaged');
hh('        daemon''s energy');
hh('dr      max MC displacement');
hh('dt      MD timestep');
hh('P       virial pressure');
hh('ar      acceptance ratio of MC displacements');
inc(aty,8);
setcolor(lightgreen);
hh('HOTKEYS');
setcolor(white);
hh('q       quiet mode (no showing)');
hh('m       toggle between Monte Carlo and molecular dynamics');
hh('e       toggle between NVE and NVT ensembles');
hh('t T - + decrease/increase temperature (NVT only)');
hh('g G     decrease/increase gravity (negative=down)');
if bc=box then hh('x X y Y toggle between repulsive (red)')
else if bc=slit then hh('y Y     toggle between repulsive (red)');
if bc<>cyclic then hh('        and attractive (white) walls');
hh('d D     MC: change displacement; red=automatic adjustment');
hh('        MD: toggle fixed timestep/automatic adjustment (red)');
hh('c C     cycle between different colors of atom trajectories');
hh('s S     decrease/increase atom size on screen');
hh('        (potential parameters do not change)');
hh('l L     decrease/increase box size');
hh('        the configuration is rescaled in the same ratio');
hh('o       restore the original box');
hh('p       toggle virial pressure calculation');
hh('n       count neighbors of atoms');
hh('cursors move the whole configuration');
hh('SPACE   freeze (pause), continue by any key');
hh('r       redraw screen');
hh('=       set more parameters');
hh('Esc     exit');
inc(aty,8);
setcolor(lightmagenta);
hh('...press any key');
if readkey=^[ then finish;
redraw;
END;

{ set some extra parameters (hotkey =) }
PROCEDURE setparms;
BEGIN
restorecrtmode;
if md then BEGIN
  h:=GetReal('integration time step dt',h);
  Tmax:=GetReal('max NVT temperature',Tmax);
  tau:=GetReal('time constant for MD friction thermostat',tau);
  END
else BEGIN
  acc:=GetReal('acceptance ratio for automatic MC displacement',acc);
  dadj:=GetReal('MC displacement (for auto)',dadj);
  END;
range:=GetReal('range for counting neighbors (FUNCTION `n'')',sqrt(range));
range:=range*range;
setgraphmode(getgraphmode);
redraw;
END;

{ to move the configuration (using cursors) }
PROCEDURE movexy(dx,dy:myreal);
BEGIN
for i:=1 to n do with r[i] do BEGIN
  x:=x+dx; if x>L then x:=x-L; if x<0 then x:=x+L;
  y:=y-dy; if y>L then y:=y-L; if y<0 then y:=y+L;
  END;
setscr;
redraw;
END;

{ count and draw neighbors (hotkey n) }
PROCEDURE neighbors;
var i,j,ix,iy:integer;
var nbr:char;
var rt:vector;
const col:array['0'..':'] of byte=(black,blue,magenta,brown,red,green,
  white,
  lightblue,lightmagenta,lightcyan,lightgray);
BEGIN
settextstyle(0,0,1);
setfillstyle(1,black);
for i:=1 to n do BEGIN
  rt:=r[i];
  nbr:='0';
  CASE bc of
    box: for j:=1 to n do if i<>j then with r[j] do
      if sqr(rt.x-x)+sqr(rt.y-y)<range then inc(nbr);
    slit: for j:=1 to n do if i<>j then with r[j] do
      if sqrni(rt.x-x)+sqr(rt.y-y)<range then inc(nbr);
    cyclic: for j:=1 to n do if i<>j then with r[j] do
      if sqrni(rt.x-x)+sqrni(rt.y-y)<range then inc(nbr);
    END;
  ix:=round(rt.x*sx)-4;
  iy:=round(rt.y*sy)-4;
  if nbr>'9' then nbr:=':';
  setcolor(col[nbr]);
  bar(ix,iy,ix+8,iy+8);
  outtextxy(ix+1,iy+1,nbr);
  END;
if readkey=^[ then finish;
redraw;
END;

{ read hotkey switch }
PROCEDURE hotkey;
BEGIN
if quiet then BEGIN
  if keypressed then BEGIN
    if readkey=^[ then finish;
    quiet:=false;
    initgraph(graphdriver,graphmode,'');
    redraw; END;
  END

else while keypressed do BEGIN

  CASE readkey of

    { quiet mode }
    'q': BEGIN
      quiet:=true;
      closegraph;
      writeln('Hi, I''m now in a quiet mode, running but not showing off');
      writeln('You can e.g. put me in the background');
      writeln('... press any key to return to graphics, ESC to quit');
      END;

    { toggle MC/MD }
    'm','M':BEGIN
      vv:=sqrt(T*2);
      md:=not md;
      if md then
        { switching to molecular dynamics: reset velocities to Maxwell-Boltzmann }
        for i:=1 to n do with v[i] do BEGIN
          x:=vv*(random-random+random-random+random-random);
          y:=vv*(random-random+random-random+random-random); END
      else BEGIN
        { switching to Monte Carlo: reset daemon's bag }
	bag:=T;
	drawcfg;
        END;
      END;

    { temperature }
    'T','+': if constt then T:=T*1.1;
    't','-': if constt then T:=T/1.1;

    { set and draw gravity }
    'g':prtgravity(-0.01);
    'G':prtgravity(0.01);

    { toggle repulsive/attractive walls }
    'x': BEGIN
      if @uwallx=@uwalla then BEGIN uwallx:=uwallr; fwallx:=fwallr; END
                         else BEGIN uwallx:=uwalla; fwallx:=fwalla; END;
       drawwalls; END;
    'X': BEGIN
      if @uwallxL=@uwalla then BEGIN uwallxL:=uwallr; fwallxL:=fwallr; END
                          else BEGIN uwallxL:=uwalla; fwallxL:=fwalla; END;
       drawwalls; END;
    'Y': BEGIN
      if @uwally=@uwalla then BEGIN uwally:=uwallr; fwally:=fwallr; END
                         else BEGIN uwally:=uwalla; fwally:=fwalla; END;
       drawwalls; END;
    'y': BEGIN
       if @uwallyL=@uwalla then BEGIN uwallyL:=uwallr; fwallyL:=fwallr; END
                            else BEGIN uwallyL:=uwalla; fwallyL:=fwalla; END;
       drawwalls; END;

    { cycle between MC displacements/MD timesteps }
    'd': if md then hauto:=not hauto
         else if dstat<fixed then inc(dstat) else dstat:=half;
    'D': if md then hauto:=not hauto
         else if dstat>half then dec(dstat) else dstat:=fixed;

    { toggle const T / const E }
    'e','E':constt:=not constt;

    { trace color }
    'c': if col=white then col:=black
         else if col=black then col:=blue
         else if col=blue then col:=white;
    'C': if col=white then col:=blue
         else if col=blue then col:=black
         else if col=black then col:=white;

    { atom size }
    's':BEGIN rad:=rad*4 div 5; redraw END;
    'S':BEGIN rad:=maxrad; redraw END;

    { box resize }
    'b','l': resize(L/1.03);
    'B','L': resize(L*1.03);
    'o','O': resize(Lorig); { original one }

    'r','R':redraw;

    { freeze }
    ' ': if readkey=^[ then finish;

    { toggle: calculate and show virial pressure }
    'p','P': BEGIN
      pressure:=not pressure;
      if not pressure then BEGIN
        fullscreen;
        info(' P=',P,darkgray,5); END;
      END;

    'n','N': neighbors;

    'h','H','?': help;

    '=': setparms;

    { move the cfg }
    #0: CASE readkey of
      'H': movexy(0,0.1+ord(bc=cyclic));
      'P': movexy(0,-0.1-ord(bc=cyclic));
      'K': movexy(-0.1-ord(bc>box),0);
      'M': movexy(0.1+ord(bc>box),0);
      END;

    ^[: finish;
    END;
  infopanel(false);
  END;
END;

{ MC or MD error }
PROCEDURE error(warning:boolean);
BEGIN
fullscreen;
setfillstyle(1,red);
settextstyle(0,0,2);
setcolor(yellow);
bar(0,0,639,100);
if warning then
  outtextxy(20,2,'Too long timestep: decreased')
else
  outtextxy(20,2,'Atom out of box or too large force');
outtextxy(20,22,'Press ENTER to continue');
outtextxy(20,42,'Press ''c'' to continue by NVT MC');
outtextxy(20,62,'Press ''='' to set parms');
outtextxy(20,82,'Press ESC to exit');
repeat ch:=readkey until ch in[^[,'c','C','=',^M];
CASE ch of
  ^[: finish;
  '=': setparms;
  ^M:;
  else BEGIN md:=false; constt:=true; END;
  END;
redraw;
END;


PROCEDURE initcfg(cfg:cfgtype);
var dn:integer;
BEGIN

Lh:=L/2;
if cfg=dump then exit;

if cfg=gas then BEGIN
  { initial random configuration }
  for i:=1 to n do with r[i] do BEGIN
    x:=0.5+random*(L-1); y:=0.5+random*(L-1);
    END;
  T:=1;
  exit;
  END;

T:=0.05;
if cfg=liquid then T:=0.15;

k:=1;
while 3*k*(k+1)+1<n do inc(k);
dec(k);
n:=3*k*(k+1)+1;
CASE cfg of
  defect: dec(n,k+1);
  vacation: dec(n);
  intersticial: inc(n);
  end;

ii:=0;
rt.x:=1.13;
if cfg=liquid then rt.x:=1.35;
rt.y:=0.866*rt.x;

for i:=k downto 0 do
  for j:=-k to k-i do BEGIN
    inc(ii);
    if (cfg=vacation) and (i=0) and (j=1) then dec(ii);
    with r[ii] do BEGIN
      x:=Lh+(i/2+j)*rt.x;
      y:=Lh+i*rt.y; END;
    if i<>0 then BEGIN
      inc(ii);
      with r[ii] do BEGIN
        x:=r[ii-1].x;
        y:=L-r[ii-1].y; END; END END;

if cfg=intersticial then with r[ii+1] do BEGIN
  x:=Lh+0.5;
  y:=Lh+0.5; END;

if cfg=liquid then ff:=1 else ff:=sqrt(T)*0.3;
for i:=1 to n do with r[i] do BEGIN
  x:=x+random*ff; y:=y+random*ff;
  END;

if cfg=defect then for i:=1 to n do with r[i] do if x>Lh then BEGIN
  ff:=sqrt((x-Lh)/k)*0.43;
  if y>Lh then BEGIN x:=x+ff; y:=y-ff END
  else BEGIN x:=x-ff; y:=y+ff END;
  END;
END;

BEGIN { ================================================================ }

bag:=0;
sweeps:=1;
pressure:=true;

randomize;

textcolor(white);
writeln;
writeln('Monte Carlo and molecular dynamics simulations of');
writeln('2D Lennard-Jones-like particles in a square simulation cell');
writeln('(c) J. Kolafa 1993-7');
writeln;
writeln('Atom-atom potential: u(r)=1/r^8-1/r^4');
writeln('Attractive atom-wall potential: uwall(r)=pi*D*(3/8/r^7-1/2/r^3)');
writeln('Repulsive atom-wall potential: uwall(r)=pi*D*3/8/r^7');
writeln;

textcolor(yellow);

if yes('special initial conditions') then BEGIN
  writeln('select initial conditions:');
  writeln('  0=gas (the default: short timestep, high Tmax)');
  writeln('  1=liquid drop (longer timestep, lower Tmax)');
  writeln('crystals (even longer timestep and lower Tmax):');
  writeln('  2=ideal crystal 3=with edge defect  4=with vacation  5=with intersticial');
  writeln('  6=read from file SHOWMCMD.DUP');
{$r+}
  cfg:=cfgtype(GetInteger('',0));
  END;

if cfg<>dump then BEGIN
  writeln('select boundary conditions:');
  bc:=bctype(GetInteger('  0=hard walls  1=slit pore (periodic in x)  2=periodic',0));
  END;
{$r-}

{ some defaults }
L:=1;
dstat:=one; dadj:=1; d:=1;

if cfg<>gas then BEGIN
  L:=0.9;
  dstat:=auto; dadj:=0.5; d:=0.5;
  h:=0.05; hadj:=0.05; Tmax:=0.3; n:=140;
  pressure:=false;
  END;

if cfg>liquid then BEGIN
  L:=0.8;
  dadj:=0.1; hadj:=0.05; d:=0.1;
  h:=0.1; Tmax:=0.16; n:=200;
  END;

if cfg<>dump then BEGIN
  n:=GetInteger('number of atoms (max 1000)',n); if (n>maxn) or (n<1) then halt;

  L:=GetReal('initial box size L',L*(sqrt(n*6)+1-ord(bc)/2));

  if bc<>cyclic then walldens:=GetReal('density of atoms on walls D',2.0);
  walldens:=walldens*pi;
  END
else
  readcfg;

Lorig:=L;

sample:=GetReal('measure sampling rate [in s]',1);
if sample<0 then sample:=0;

uwallx:=uwallr; fwallx:=fwallr;
uwally:=uwallr; fwally:=fwallr;
uwallxL:=uwallr; fwallxL:=fwallr;
uwallyL:=uwallr; fwallyL:=fwallr;
gravity:=0;

col:=black;

{T:=GetReal('initial temperature T',1);}


initcfg(cfg);

{ starting graphics }
graphdriver:=detect;
initgraph(graphdriver,graphmode,'');

getaspectratio(aspx,aspy);
sy:=getmaxy/Lorig;
sx:=sy*aspy/aspx;
maxrad:=trunc(sx/2.0);
rad:=maxrad;

setscr;

md:=false; constt:=true;
ctime;
redraw;

repeat

time0:=time;

if col>15 then col:=0;
if col<0 then col:=15;

Tm:=0;
accepted:=0;

for ii:=1 to sweeps do BEGIN

  if not quiet then BEGIN
    fullscreen;
    bar(getmaxx-32,0,getmaxx,16);
    setcolor(lightblue);
    outtextxy(getmaxx-32,0,chr((ii div 10) mod 10+48)+chr(ii mod 10+48));
    insquare; END;

  if md then BEGIN { ===== molecular dynamics ===== }

    if hauto then h:=hadj;

    { calculate forces (accelerations) caused by walls and gravity }
    for i:=1 to n do with a[i] do
      if bc=cyclic then BEGIN
        x:=0; y:=0; END
      else BEGIN
        ri:=r[i];
        y:=fwally(ri.y)-fwallyL(L-ri.y)-gravity;
        if bc=box then x:=fwallx(ri.x)-fwallxL(L-ri.x)
        else x:=0; END;

    { calculate atom-atom pair forces }
    CASE bc of
      box: for i:=1 to n do with a[i] do BEGIN
        ri:=r[i];
        for j:=i+1 to n do with r[j] do BEGIN
          rt.x:=ri.x-x; rt.y:=ri.y-y;
          ff:=f(sqr(rt.x)+sqr(rt.y));
          with a[i] do BEGIN x:=x+rt.x*ff; y:=y+rt.y*ff; END;
          with a[j] do BEGIN x:=x-rt.x*ff; y:=y-rt.y*ff; END; END;
        END;
      slit: for i:=1 to n do with a[i] do BEGIN
        ri:=r[i];
        for j:=i+1 to n do with r[j] do BEGIN
          rt.x:=ri.x-x;
          if rt.x>Lh then rt.x:=rt.x-L;
          if rt.x<-Lh then rt.x:=rt.x+L;
          rt.y:=ri.y-y;
          ff:=f(sqr(rt.x)+sqr(rt.y));
          with a[i] do BEGIN x:=x+rt.x*ff; y:=y+rt.y*ff; END;
          with a[j] do BEGIN x:=x-rt.x*ff; y:=y-rt.y*ff; END; END;
        END;
      cyclic: for i:=1 to n do with a[i] do BEGIN
        ri:=r[i];
        for j:=i+1 to n do with r[j] do BEGIN
          rt.x:=ri.x-x;
          if rt.x>Lh then rt.x:=rt.x-L;
          if rt.x<-Lh then rt.x:=rt.x+L;
          rt.y:=ri.y-y;
          if rt.y>Lh then rt.y:=rt.y-L;
          if rt.y<-Lh then rt.y:=rt.y+L;
          ff:=f(sqr(rt.x)+sqr(rt.y));
          with a[i] do BEGIN x:=x+rt.x*ff; y:=y+rt.y*ff; END;
          with a[j] do BEGIN x:=x-rt.x*ff; y:=y-rt.y*ff; END; END;
        END;
      END;

    { leap-frog integrator, calculate kinetic energy, and draw atoms }
    vv:=0;
    maxvv:=0;

    for i:=1 to n do with r[i] do with scr[i] do BEGIN

      { crash integrator test }
      with a[i] do if (abs(x)>1000) or (abs(y)>1000) then BEGIN
        error(false);
        goto escape; END;

      { leap-frog }
      with v[i] do BEGIN
        x:=x+h*a[i].x; y:=y+h*a[i].y;
        xx:=sqr(x)+sqr(y);
        vv:=vv+xx;  { Ekin is calculated at time t+h/2 }
        if xx>maxvv then maxvv:=xx;
        END;
      x:=x+h*v[i].x;  y:=y+h*v[i].y;

      if not quiet then BEGIN
        if repaint then BEGIN
          setcolor(col);
          circle(ix,iy,rad); END;
        ix:=round(x*sx); iy:=round(y*sy);
        setcolor(white);
        circle(ix,iy,rad); END;
      END;

    maxvv:=sqrt(maxvv);
    xx:=maxvv*h;
    if xx>0.15 then BEGIN
      h:=0.06/sqrt(maxvv);
      hadj:=h;
      error(true); END;
    if xx>0.12 then hadj:=0.1/maxvv
    else hadj:=hadj*0.95+(0.09*0.05)/maxvv;
    if hadj>0.1 then hadj:=0.1;

    vv:=vv/2; { Ekin }
    Tk:=vv/n; { kinetic temperature }

    if constt then BEGIN
      { friction thermostat: rescale velocities, correlation time ~ tau }
      ff:=exp((T-Tk)*h/tau);
      for i:=1 to n do with v[i] do BEGIN x:=x*ff; y:=y*ff; END; END;

    Tm:=Tm+vv;

    END { MD }

  else BEGIN { ===== Monte Carlo ===== }

    { max displacement = Lh=L/2 }
    if dadj>Lh then dadj:=Lh;

    { determining the current displacement }
    CASE dstat of
      half: d:=0.5;
      one:  d:=1;
{      two:  d:=2;}
      Lhalf: d:=L/2;
      else d:=dadj; END;

    for i:=1 to n do BEGIN

      { trial move of atom i }

      ri:=r[i]; { old position }

      { random displacement in a 1-circle }
      repeat
        incirc.x:=2*random-1;
        incirc.y:=2*random-1;
        until sqr(incirc.x)+sqr(incirc.y)<1;

      { new (trial) position (not too much efficient but simple) }
      rt.x:=ri.x+incirc.x*d;

      { normalize to the box, even for box+slit b.c. which prevents out-of-box problems }
      if rt.x<0 then rt.x:=rt.x+L
      else if rt.x>L then rt.x:=rt.x-L;

      { ... and the same for y coord }
      rt.y:=ri.y+incirc.y*d;
      if rt.y<0 then rt.y:=rt.y+L
      else if rt.y>L then rt.y:=rt.y-L;

      { energy difference caused by the trial move: walls and gravity }
      deltaU:=0;
      if bc<=slit then BEGIN
         deltaU:=uwally(rt.y)-uwally(ri.y)
                +uwallyL(L-rt.y)-uwallyL(L-ri.y)
                -gravity*(ri.y-rt.y);
         if bc=box then deltaU:=deltaU+
           uwallx(rt.x)-uwallx(ri.x)
          +uwallxL(L-rt.x)-uwallxL(L-ri.x);
         END;

      { : all other atoms ... not efficient, old energies calculated
          again and again though they might have been stored }
      CASE bc of
        box: for j:=1 to n do if i<>j then with r[j] do
          deltaU:=deltaU
            +u(sqr(rt.x-x)+sqr(rt.y-y))-u(sqr(ri.x-x)+sqr(ri.y-y));
        slit: for j:=1 to n do if i<>j then with r[j] do
          deltaU:=deltaU
            +u(sqrni(rt.x-x)+sqr(rt.y-y))-u(sqrni(ri.x-x)+sqr(ri.y-y));
        cyclic: for j:=1 to n do if i<>j then with r[j] do
          deltaU:=deltaU
            +u(sqrni(rt.x-x)+sqrni(rt.y-y))-u(sqrni(ri.x-x)+sqrni(ri.y-y));
        END;

      if constt and (-deltaU/T>ln(random)) { Metropolis test }
      or not constt and (deltaU<bag) { Creutz microcanonical test }
      then with scr[i] do BEGIN
        { move accepted }
        if not constt then bag:=bag-deltaU;
        inc(accepted);
        r[i]:=rt;

        if not quiet then BEGIN
          if repaint then BEGIN
            setcolor(col);
            circle(ix,iy,rad); END;
          setcolor(white);
          with r[i] do BEGIN ix:=round(x*sx); iy:=round(y*sy); END;
          circle(ix,iy,rad);
          END;

       { automatic adjustment of displacement size to reach
         acceptance ratio = acc; tiny violation of
         microreversibility is usually acceptable }
        if dstat=auto then dadj:=dadj*(1+(1-acc)*1e-3);
        END
      else
        { move rejected }
        if dstat=auto then dadj:=dadj*(1-acc*1e-3); { automatic adjustment of displacement size }

      Tm:=Tm+bag;
      END;
    END; {MC}

  { fool-proof out-of-box check }
  escape:
  for i:=1 to n do with r[i] do BEGIN
    if (x<=0) or (x>=L) then BEGIN
      if (not md) or (bc=box) then error(false);
      while x<0 do x:=x+L;
      while x>L do x:=x-L; END;
    if (y<=0) or (y>=L) then BEGIN
      if (not md) or (bc<>cyclic) then error(false);
      while y<0 do y:=y+L;
      while y>L do y:=y-L; END;
    END;

  hotkey;

  repaint:=col<>white;

  END; {sweeps}

Tm:=Tm/(n*sweeps);
ar:=accepted/(n*sweeps);

{ calculate virial pressure }
if pressure then BEGIN
  P:=0; { P=sum r*f = -virial}
  for i:=1 to n do BEGIN
    ri:=r[i];
    if bc<cyclic then BEGIN
      P:=P+ri.y*fwally(ri.y)+(L-ri.y)*fwallyL(L-ri.y);
      if bc=box then P:=P+ri.x*fwallx(ri.x)+(L-ri.x)*fwallxL(L-ri.x)
      END;
    CASE bc of
      box: for j:=i+1 to n do with r[j] do BEGIN
        rr:=sqr(ri.x-x)+sqr(ri.y-y);
        P:=P+rr*f(rr); END;
      slit: for j:=i+1 to n do with r[j] do BEGIN
        rr:=sqrni(ri.x-x)+sqr(ri.y-y);
        P:=P+rr*f(rr); END;
      cyclic: for j:=i+1 to n do with r[j] do BEGIN
        rr:=sqrni(ri.x-x)+sqrni(ri.y-y);
        P:=P+rr*f(rr); END;
      END;
    END;
  if md then TT:=Tk else TT:=T;
  P:=(n*TT+P/3)/sqr(L);
  fullscreen;
  info(' P=',P,lightcyan,5);
  END;

infopanel(true);

{ changes # of sweeps / 1 measurement cycle according to real execution time }
if sample>0 then BEGIN
  ctime;
  ff:=sample/(time-time0+0.01);
  if ff<0.5 then ff:=0.5;
  if ff>2 then ff:=2;
  sweeps:=round(sweeps*ff);
  if sweeps<=0 then sweeps:=1;
  END;

until false;

END.
